diff -ru zip/reader.go zzz/reader.go
--- zip/reader.go	2022-04-12 11:42:57.000000000 -0400
+++ zzz/reader.go	2022-04-20 13:45:07.132655968 -0400
@@ -55,6 +55,7 @@
 	headerOffset int64
 	zip64        bool  // zip64 extended information extra field presence
 	descErr      error // error reading the data descriptor during init
+	filestart    int64
 }
 
 // OpenReader will open the Zip file specified by name and return a ReadCloser.
@@ -107,7 +108,7 @@
 	}
 	z.Comment = end.comment
 	rs := io.NewSectionReader(r, 0, size)
-	if _, err = rs.Seek(int64(end.directoryOffset), io.SeekStart); err != nil {
+	if _, err = rs.Seek(end.filestart+int64(end.directoryOffset), io.SeekStart); err != nil {
 		return err
 	}
 	buf := bufio.NewReader(rs)
@@ -118,7 +119,7 @@
 	// the file count modulo 65536 is incorrect.
 	for {
 		f := &File{zip: z, zipr: r}
-		err = readDirectoryHeader(f, buf)
+		err = readDirectoryHeader(f, buf, end.filestart)
 		if err == ErrFormat || err == io.ErrUnexpectedEOF {
 			break
 		}
@@ -254,6 +255,16 @@
 	return headerFileInfo{&r.f.FileHeader}, nil
 }
 
+func (r *checksumReader) Seek(offset int64, whence int) (int64, error) {
+	if whence == io.SeekEnd {
+		return int64(r.f.FileHeader.UncompressedSize), nil
+	}
+	r2, _ := r.f.Open()
+	rr := r2.(*checksumReader)
+	*r = *rr
+	return 0, nil
+}
+
 func (r *checksumReader) Read(b []byte) (n int, err error) {
 	if r.err != nil {
 		return 0, r.err
@@ -297,7 +308,7 @@
 // and returns the file body offset.
 func (f *File) findBodyOffset() (int64, error) {
 	var buf [fileHeaderLen]byte
-	if _, err := f.zipr.ReadAt(buf[:], f.headerOffset); err != nil {
+	if _, err := f.zipr.ReadAt(buf[:], f.filestart+f.headerOffset); err != nil {
 		return 0, err
 	}
 	b := readBuf(buf[:])
@@ -307,13 +318,13 @@
 	b = b[22:] // skip over most of the header
 	filenameLen := int(b.uint16())
 	extraLen := int(b.uint16())
-	return int64(fileHeaderLen + filenameLen + extraLen), nil
+	return f.filestart + int64(fileHeaderLen+filenameLen+extraLen), nil
 }
 
 // readDirectoryHeader attempts to read a directory header from r.
 // It returns io.ErrUnexpectedEOF if it cannot read a complete header,
 // and ErrFormat if it doesn't find a valid header signature.
-func readDirectoryHeader(f *File, r io.Reader) error {
+func readDirectoryHeader(f *File, r io.Reader, filestart int64) error {
 	var buf [directoryHeaderLen]byte
 	if _, err := io.ReadFull(r, buf[:]); err != nil {
 		return err
@@ -339,6 +350,7 @@
 	b = b[4:] // skipped start disk number and internal attributes (2x uint16)
 	f.ExternalAttrs = b.uint32()
 	f.headerOffset = int64(b.uint32())
+	f.filestart = filestart
 	d := make([]byte, filenameLen+extraLen+commentLen)
 	if _, err := io.ReadFull(r, d); err != nil {
 		return err
@@ -565,6 +577,7 @@
 		directoryOffset:    uint64(b.uint32()),
 		commentLen:         b.uint16(),
 	}
+	d.filestart = directoryEndOffset - int64(d.directorySize+d.directoryOffset)
 	l := int(d.commentLen)
 	if l > len(b) {
 		return nil, errors.New("zip: invalid comment length")
diff -ru zip/struct.go zzz/struct.go
--- zip/struct.go	2022-04-12 11:42:57.000000000 -0400
+++ zzz/struct.go	2022-04-20 13:45:07.132655968 -0400
@@ -197,6 +197,7 @@
 	directoryRecords   uint64
 	directorySize      uint64
 	directoryOffset    uint64 // relative to file
+	filestart          int64
 	commentLen         uint16
 	comment            string
 }
